/*
								+--------------------------------+
								|                                |
								|    ***   File listener   ***   |
								|                                |
								|  Copyright  -tHE SWINe- 2008  |
								|                                |
								|           FileList.h           |
								|                                |
								+--------------------------------+
*/

#ifndef __FILE_LISTENER_INCLUDED
#define __FILE_LISTENER_INCLUDED

/*
 *	class CFileListener
 *		- file listener for directory traversal
 *		- records all files to std::vector<TFileInfo*>
 */
class CFileListener {
protected:
	std::vector<TFileInfo*> &m_r_file_list;
	uint64_t m_n_total_size;
	bool m_b_size_valid;
	bool m_b_success;

public:
	/*
	 *	CFileListener::CFileListener(std::vector<TFileInfo*> &r_file_list)
	 *		- default constructor; sets output file list (should be empty)
	 *		- note the list will contain dynamically allocated data,
	 *		  make sure to delete it, for example using Delete_FileList
	 */
	CFileListener(std::vector<TFileInfo*> &r_file_list);

	/*
	 *	void CFileListener::operator ()(const TFileInfo &r_file_info)
	 *		- executed by CDirTraversal, adds a new file to the list
	 *		- this function may actually fail (in case there's not enough memory)
	 *		  so converting this class to bool gives it's status
	 *		- in case this fails, file list is guaranteed to be empty
	 */
	void operator ()(const TFileInfo &r_file_info);

	/*
	 *	uint64_t CFileListener::n_Total_Size() const
	 *		- returns total size of all files in the list
	 *		- note the size might be actually greater than maximum value of uint64_t
	 *		  so there's function bool b_Size_Valid() which tells wheter value returned
	 *		  by this function is valid
	 */
	uint64_t n_Total_Size() const;

	/*
	 *	bool CFileListener::b_Size_Valid() const
	 *		- if value returned by n_Total_Size() is valid, returns true.
	 *		  otherwise (overflow occured) returns false
	 */
	bool b_Size_Valid() const;

	/*
	 *	operator CFileListener::bool() const
	 *		- in case there was not enough memory for all files in the
	 *		  file list, returns false. otherwise returns true.
	 */
	operator bool() const;

	/*
	 *	static inline void CFileListener::Delete_FileList(std::vector<TFileInfo*> &r_file_list)
	 *		- utility cleanup function
	 */
	static void Delete_FileList(std::vector<TFileInfo*> &r_file_list);

protected:
	static inline void DeleteFileInfo(TFileInfo *p_file_info);
	bool ProcessFile(const TFileInfo &r_file_info);
	static bool CheckRead(const char *p_s_filename);
};

#endif //__FILE_LISTENER_INCLUDED
